#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

'''

Needs Workspace Fixes

class WorkspaceTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("WorkspaceTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)


    def test_pull(self):

        self.logger.info("Testing Workspace Pull")

        test_obj = abobjs.AB_Workspace(control_id=self.test_config["test"]["read_workspace_id"],
                                       api_info=self.test_config["api"])

        self.assertTrue(isinstance(test_obj.id, int))


'''


import logging
import urllib.parse
import datetime
import json

import requests
import pyjq

import abobjs

class AB_Entity(abobjs.AB_Base):

    """
    A.K.A. Cycles. Not Sure why the mismatch exists
    """

    _endpoint = "/api/v1/entities"
    _single = ".entities[0]"
    _name = "entity"
    #_datefront = "%Y-%m-%dT%H:%M:%S"
    #_mslength = 3
    #_date_keys = ["created_at", "updated_at", "deleted_at",	"effective_date",
	#              "baseline_date", "last_modification_date", "last_review_date"]

    _uid_field = "entity_code"

    def __init__(self, id=None, api_info=None, **kwargs):

        #self.api_info = api_info
        #self.id = control_id
        #self.datum = kwargs.get("control_datum", dict())

        # Dunder Handling
        endpoint = kwargs.get("endpoint", self._endpoint)
        single = kwargs.get("single", self._single)
        name = kwargs.get("name", self._name)
        uid_field = kwargs.get("uid_field", self._uid_field)

        strips = kwargs.get("strips", self._strips)

        abobjs.AB_Base.__init__(self, id=id, api_info=api_info,
                                endpoint=endpoint, single=single, name=name, strips=strips,
                                uid_field=uid_field,
                                **kwargs)

        if kwargs.get("control_ids", None)  is not None:
            self.control_link = self.set_controls(control_ids=kwargs["control_ids"])

    def set_controls(self, control_ids=list()):

        if len(control_ids) == 0 or isinstance(control_ids, list) is False:
            raise ValueError("We need to set a list of controls.")

        add_controls_endpoint = urllib.parse.urljoin(self.api_info.get("base_domain", None),
                                                    "{}/{}/add_controls".format(self.endpoint, self.id))

        post_data = self.serialize(full_json=False, cust_data={"entityId": self.id, "control_ids":  control_ids})

        try:
            data_request = requests.post(add_controls_endpoint, headers=self.headers, json=post_data)
        except Exception as error:
            self.logger.error("Error when Adding Controls {} to Entity: {}".format(control_ids, self.id))
            self.logger.debug("Error Message: {}".format(error))
            objectified_data = self.objectify_datum({"entityId": self.id, "control_ids": []})
        else:
            self.logger.debug("Create Response: {}".format(data_request.status_code))
            if data_request.status_code != requests.codes.ok:
                self.logger.debug(data_request.text)
                raise ValueError("{} Add Control erorr {}".format(self.name, data_request.text))

            objectified_data = self.objectify_datum(data_request.json())

        return objectified_data


#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

class RegionTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("RegionTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)


    def test_pull(self):

        self.logger.info("Testing Region Pull")

        test_obj = abobjs.AB_Region(id=self.test_config["test"]["read_region_id"],
                                    api_info=self.test_config["api"])

        #self.logger.debug(test_obj.datum)
        self.assertTrue(isinstance(test_obj.id, int))

    def test_create_delete(self):

        sample_control = {'region_code': 'CRTC',
                          'name': 'Control Rod Test Cycle',
                          'workspace_id': 1}

        create_obj = abobjs.AB_Region(api_info=self.test_config["api"],
                                      datum=sample_control,
                                      init_action="create")

        self.assertTrue(isinstance(create_obj.id, int))
        self.assertTrue(isinstance(create_obj.datum, dict))

        create_obj.datum["name"] = "Changed Name"
        create_obj.update()
        create_obj.get()

        self.assertEqual(create_obj.datum["name"], "Changed Name")

        create_obj.delete()



#!/usr/bin/env python3

import logging
import urllib.parse
import datetime
import json

import requests
import pyjq

import abobjs

class AB_Controls_Data(abobjs.AB_Base):

    _endpoint = "/api/v1/controls_data"
    _single = ".controls_data[0]"
    _name = "controls_data"
    _file_modal = "controlsDatum"
    _fileable_type = "ControlsDatum"

    _date_keys = [*abobjs.AB_Base._date_keys]

    _create_null_keys = [*abobjs.AB_Base._create_null_keys, "xnull"]

    _read_args = {"include[]": ["tests", "files"]}


    def __init__(self, id=None, api_info=None, **kwargs):

        # Dunder Handling
        endpoint = kwargs.get("endpoint", self._endpoint)
        single = kwargs.get("single", self._single)
        name = kwargs.get("name", self._name)

        date_keys = kwargs.get("date_keys", self._date_keys)
        create_null_keys = kwargs.get("create_null_keys", self._create_null_keys)

        file_modal = kwargs.get("file_modal", self._file_modal)
        fileable_type = kwargs.get("fileable_type", self._fileable_type)

        abobjs.AB_Base.__init__(self, id=id, api_info=api_info,
                                endpoint=endpoint, single=single, name=name,
                                date_keys=date_keys,
                                create_null_keys=create_null_keys,
                                file_modal=file_modal,
                                fileable_type=fileable_type,
                                **kwargs)



#!/usr/bin/env python3
import io
import unittest
import importlib.resources
import logging
import uuid

import ci
import yaml

import abobjs

class TestFiles(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("TestsTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)


    def test_pull(self):

        self.logger.info("Testing File Pull Via Controls Datum Pull")

        #cd_obj = abobjs.AB_Controls_Data(id=self.test_config["test"]["read_controls_datum_id"],
        #                             api_info=self.test_config["api"])

        cd_obj = abobjs.AB_Controls_Data(id=1187,
                                         api_info=self.test_config["api"])


        # Upload A File
        with io.StringIO() as sample_fobj:
            sample_fobj.write("This is a Sample Test File Created")
            sample_fobj.seek(0)

            cd_obj.upload_file(sample_fobj, "{}.txt".format(str(uuid.uuid4())))

            cd_obj.datum["test_ids"][0].upload_file(sample_fobj, "{}.txt".format(str(uuid.uuid4())))

        self.logger.info(cd_obj.datum)

        for this_file in cd_obj.datum["file_ids"]:
            # Do A Forward back
            self.logger.info("Files ID: {}".format(this_file.id))


        self.assertTrue(isinstance(cd_obj.id, int))



import logging
import urllib.parse
import datetime
import json

import requests
import pyjq

import abobjs

class AB_Risk(abobjs.AB_Base):

    """
    A.K.A. Cycles. Not Sure why the mismatch exists
    """

    _endpoint = "/api/v1/risks"
    _single = ".risks[0]"
    _name = "risk"
    #_datefront = "%Y-%m-%dT%H:%M:%S"
    #_mslength = 3
    _date_keys = ["custom_date1", "custom_date2", "custom_date3", "custom_date4",
                  *abobjs.AB_Base._date_keys]

    #_strips = ["form_templates", "sort_order", "enabled_attributes", "excluded_attributes"]

    def __init__(self, id=None, api_info=None, **kwargs):

        #self.api_info = api_info
        #self.id = control_id
        #self.datum = kwargs.get("control_datum", dict())

        # Dunder Handling
        endpoint = kwargs.get("endpoint", self._endpoint)
        single = kwargs.get("single", self._single)
        name = kwargs.get("name", self._name)
        date_keys = kwargs.get("date_keys", self._date_keys)

        #print(self._strips)
        #strips = kwargs.get("strips", self._strips)

        abobjs.AB_Base.__init__(self, id=id, api_info=api_info,
                                endpoint=endpoint, single=single, name=name,
                                date_keys=date_keys,
                                **kwargs)


#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

class TestTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("TestsTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)


    def test_pull(self):

        self.logger.info("Testing Test Pull Via Controls Datum Pull")

        cd_obj = abobjs.AB_Controls_Data(id=self.test_config["test"]["read_controls_datum_id"],
                                     api_info=self.test_config["api"])

        for this_test in cd_obj.datum["test_ids"]:
            # Do A Forward back
            string_var = "put in test suite\n"
            this_test.datum["results"] = string_var

            this_test.update()

            this_test.get()

            self.assertEqual(this_test.datum["results"], string_var)

            this_test.datum["results"] = "\n"

            this_test.update()
            this_test.get()

            self.logger.warning(this_test.datum)

            self.assertEqual(this_test.datum["results"], "\n")
            break

        self.assertTrue(isinstance(cd_obj.id, int))



#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

class ControlRodTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("ControlRodTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)

        with importlib.resources.path(ci, "sample_rod.yaml") as test_cr_path_gen:
            test_cr = test_cr_path_gen

        with open(test_cr, "r") as cr_fobj:
            self.cr_config = yaml.safe_load(cr_fobj)


    def test_rod_create_pull(self):

        self.logger.info("Testing Control Create")

        control_rod = abobjs.AB_ControlRod(cr_data=self.cr_config,
                                           api_info=self.test_config["api"])

        self.assertTrue(isinstance(control_rod.roc_objs["region"], list))

        controls_data = abobjs.AB_Controls_Data(api_info=self.test_config["api"],
                                                uid="EA.CRP.S1.CR1")

        print(controls_data.datum)





#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

class SubProcessTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("SubProcessTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)


    def test_pull(self):

        self.logger.info("Testing SubProcess Pull")

        test_obj = abobjs.AB_SubProcess(id=self.test_config["test"]["read_subprocess_id"],
                                        api_info=self.test_config["api"])

        self.logger.debug(test_obj.datum)
        self.assertTrue(isinstance(test_obj.id, int))

    '''
    # Add object Serialization/Deserialization
    
    def test_create_delete(self):

        sample_process = {
            "uid": "CRTP",
            "name": "Control Rod Test Process",
            "process_type_ids": []
        }

        create_obj = abobjs.AB_SubProcess(api_info=self.test_config["api"],
                                          datum=sample_process,
                                          init_action="create")

        self.assertTrue(isinstance(create_obj.id, int))
        self.assertTrue(isinstance(create_obj.datum, dict))

        create_obj.datum["name"] = "Changed Name"
        create_obj.update()
        create_obj.get()

        self.assertEqual(create_obj.datum["name"], "Changed Name")

        create_obj.delete()
    '''



#!/usr/bin/env python3

import logging
import pyjq


class AB_Multi:
    '''
    Hit an endpoint for a given type and give me a container with all of the objects in that list

    Useful for "give me all the controls".
    '''

    def __init__(self, type=None, api_info=None, filter=None, **kwargs):

        self.logger = logging.getLogger("multi")

        self.api_info = api_info
        self.type = type
        self.filter = filter
        self.metadata = dict(results=0, errors=list())
        self.multidatum = kwargs.get("multidatum", list())

        self.kwargs = kwargs

        self.base_search_obj = self.type(api_info=self.api_info,
                                         multi=True,
                                         max_depth=self.kwargs.get("max_depth", -1),
                                         read_args=self.kwargs.get("read_args", {}))

        for obj in pyjq.all(self.base_search_obj.all_jq, self.base_search_obj.kwargs["all_data"]):

            this_okay = True

            # Future Checks for Okayness based on Filters

            if this_okay is True:
                this_id = obj[self.base_search_obj.id_field]

                this_obj = self.type(id=this_id, api_info=self.api_info)

                self.metadata["results"] += 1
                self.multidatum.append(this_obj)

    def check_filter(self, object):

        okay = True

        # ADD PFE Logic here in the Future

        return okay



from .base import AB_Base

from .control import AB_Control
from .workspace import AB_Workspace
from .region import AB_Region
from .entity import AB_Entity
from .process import AB_Process
from .subprocess import AB_SubProcess
from .risk import AB_Risk
from .assertion import AB_Assertion
from .test_section import AB_TestSection
from .controls_data import AB_Controls_Data
from .effectiveness_option import AB_EffectivenessOption
from .status_option import AB_StatusOption
from .test_type import AB_TestType
from .test import AB_Test
from .file import AB_File

from .continuous_monitoring_systems import AB_Continous_Monitoring_Systems
from .continuous_monitoring_monitors import AB_Continous_Monitoring_Monitors
from .continuous_monitoring_monitor_results import AB_Continous_Monitoring_Monitor_Results

from .auditable_entity import AB_Auditable_Entities
from .auditable_entity_regions import AB_Auditable_Entity_Regions
from .auditable_entity_types import AB_Auditable_Entity_Types
from .vendor_criticalities import AB_Vendor_Criticality

from .buisness_units import AB_Business_Unit
from .departments import AB_Departments

from .multi import AB_Multi

SUBOBJ_LK = {"region_id": {"obj": AB_Region},
             "process_id": {"obj": AB_Process},
             "subprocess_id": {"obj": AB_SubProcess},
             "risk_id": {"obj": AB_Risk},
             "control_id": {"obj": AB_Control},
             "entity_id": {"obj": AB_Entity},
             "controls_datum_ids": {"obj": AB_Controls_Data},
             "test_ids": {"obj": AB_Test},
             "file_ids": {"obj": AB_File},
             "continuous_monitoring_system_id": {"obj": AB_Continous_Monitoring_Systems},
             "continuous_monitoring_monitor_id": {"obj": AB_Continous_Monitoring_Monitors},
             "auditable_entity_id": {"obj": AB_Auditable_Entities},
             "auditable_entity_region_id": {"obj": AB_Auditable_Entity_Regions},
             "vendor_criticalities_id": {"obj": AB_Vendor_Criticality}
            }


from .controlrod import AB_ControlRod

USER_AGENT = "control_rod/0.0.0"


#!/usr/bin/env python3

import unittest
import importlib.resources
import logging

import ci
import yaml

import abobjs

class ControlsDatumTests(unittest.TestCase):

    @classmethod
    def setUp(self):

        self.logger = logging.getLogger("ControlsDatumTests")

        with importlib.resources.path(ci, "config.yaml") as test_config_path_gen:
            test_config_path = test_config_path_gen

        with open(test_config_path, "r") as test_config_fobj:
            self.test_config = yaml.safe_load(test_config_fobj)

    def test_pull(self):

        self.logger.info("Testing Controls Datum Pull")

        test_obj = abobjs.AB_Controls_Data(id=self.test_config["test"]["read_controls_datum_id"],
                                     api_info=self.test_config["api"])

        self.assertTrue(isinstance(test_obj.id, int))

    '''
    def test_create_delete(self):

        self.logger.info("Testing Controls Datum Create/Delete")

        sample_control = {"name": "Controls Datum Test",
                          "uid": "DATUM3",
                          "subprocess_id": self.test_config["test"]["read_subprocess_id"]
                          }

        create_obj = abobjs.AB_Controls_Data(api_info=self.test_config["api"],
                                             datum=sample_control,
                                             init_action="create")

        self.assertTrue(isinstance(create_obj.id, int))
        self.assertTrue(isinstance(create_obj.datum, dict))

        search_obj = abobjs.AB_Controls_Data(api_info=self.test_config["api"],
                                             init_action="read",
                                             uid=sample_control["uid"])

        self.assertEqual(create_obj.id, search_obj.id)

        create_obj.datum["name"] = "Changed Name"
        create_obj.update()
        create_obj.get()

        self.assertEqual(create_obj.datum["name"], "Changed Name")

        print(create_obj.datum)
        create_obj.delete()

    '''


#!/usr/bin/env python3

import logging
import urllib.parse
import datetime
import json

import requests
import pyjq

import abobjs

class AB_Auditable_Entity_Types(abobjs.AB_Base):

    _endpoint = "/api/v1/auditable_entity_types"
    _single = ".auditable_entity_types[0]"
    _name = "auditable_entity_type"
    _uid_field = "key"

    def __init__(self, id=None, api_info=None, **kwargs):

        # Dunder Handling
        endpoint = kwargs.get("endpoint", self._endpoint)
        single = kwargs.get("single", self._single)
        name = kwargs.get("name", self._name)

        abobjs.AB_Base.__init__(self, id=id, api_info=api_info,
                                endpoint=endpoint, single=single, name=name,
                                **kwargs)
